import java.io.IOException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

public class XPathConstantsExample {

	public static void main(String[] args) {

		DocumentBuilderFactory factory;
		DocumentBuilder builder;
		try {
			// Create DocumentBuilderFactory for reading xml file
			factory = DocumentBuilderFactory.newInstance();
			builder = factory.newDocumentBuilder();
			Document doc = builder.parse("employee.xml");

			// Create XPathFactory for creating XPath Object
			XPathFactory xPathFactory = XPathFactory.newInstance();
			// Create XPath object from XPathFactory
			XPath xpath = xPathFactory.newXPath();

			// Method to demonstrate various XPathConstants
			demonstrateConstants(doc, xpath);

		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		} catch (SAXException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Method to demonstrate various XPathConstants
	 * 
	 * @param doc
	 * @param xpath
	 */
	private static void demonstrateConstants(Document doc, XPath xpath) {
		try {
			// 1. XPathConstants.STRING
			// Compile the XPath expression for getting a string value
			XPathExpression xPathExpr = xpath.compile("/Employees/Employee/name/text()");
			// XPath text example : executing xpath expression in java Object
			Object result = xPathExpr.evaluate(doc, XPathConstants.STRING);
			System.out.println("Java XpathConstants String example:::::" + result);

			// 2. XPathConstants.NUMBER
			// Compile the XPath expression for getting a number value
			xPathExpr = xpath.compile("/Employees/Employee/age");
			result = xPathExpr.evaluate(doc, XPathConstants.NUMBER);
			System.out.println("Java XpathConstants Number example:::::" + result);

			// 3. XPathConstants.BOOLEAN
			// Compile the XPath expression for getting a boolean value
			xPathExpr = xpath.compile("/Employees/Employee/fulltime");
			result = xPathExpr.evaluate(doc, XPathConstants.BOOLEAN);
			System.out.println("Java XpathConstants Boolean example:::::" + result);

			// 4. XPathConstants.NODESET
			// Compile the XPath expression for getting a NodeSet: Gets all name tag values from employee.xml	
			System.out.println("Java XpathConstants NODESET example:::::getting all nodes name attribute");
			xPathExpr = xpath.compile("/Employees/Employee/name/text()");			
			result = xPathExpr.evaluate(doc, XPathConstants.NODESET);
			printXpathResult(result);

			// Compile the XPath expression for getting a NodeSet: Gets all age tag values from employee.xml
			System.out.println("Java XpathConstants NODESET example:::::getting all nodes age attribute");
			xPathExpr = xpath.compile("/Employees/Employee/age/text()");
			result = xPathExpr.evaluate(doc, XPathConstants.NODESET);
			printXpathResult(result);
			
			// Compile the XPath expression for getting a NodeSet: Gets all gender tag values from employee.xml
			System.out.println("Java XpathConstants NODESET example:::::getting all nodes gender attribute");
			xPathExpr = xpath.compile("/Employees/Employee/gender/text()");
			result = xPathExpr.evaluate(doc, XPathConstants.NODESET);
			printXpathResult(result);
			
			// Compile the XPath expression for getting a NodeSet: Gets all fulltime tag values from employee.xml
			System.out.println("Java XpathConstants NODESET example:::::getting all nodes fulltime attribute");
			xPathExpr = xpath.compile("/Employees/Employee/fulltime/text()");
			result = xPathExpr.evaluate(doc, XPathConstants.NODESET);
			printXpathResult(result);

		} catch (XPathExpressionException e) {
			e.printStackTrace();
		}
	}
	
	/**
	* Method to print result on console
	* @param result
	*/
	
	public static void printXpathResult(Object result) {
	   NodeList nodes = (NodeList) result;
	   for (int i = 0; i < nodes.getLength(); i++) {
		   System.out.println(nodes.item(i).getNodeValue());
	   }
	}
}
